﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using DataFlowConverter;

namespace CodeRetrieverDataflow
{
    public partial class Form1 : Form
    {
        // This file holds all of the global vars used by the CodeRetriever application.

        /*
         * 11/14/18 NOTE: 'convertRubyToJS' mode has been disabled since we are converting from
         * SuperBoxing using Innosphere's converter. However, the Retriever's converter does function.
         * It identifies current Retriever Ruby snippets using a hashing algorithm. Older Ruby
         * snippets can also be identified but those older snippets would need to be hashed and the
         * hash added to the appropriate snippet's hash ['|' + hash number]. The hash can be calculated
         * by using the tool ConvertRuby_To_JS, a Visual Studio C# Form application.
         * 
         * These are our 3 modes of operation:
         * 
         * prevRetrieveMode holds either 'retrieveRuby' or 'retrieveJS', we set it when moving
         * into 'convertRubyToJS' mode. When we return to either 'retrieveRuby' or 'retrieveJS'
         * we check to see if prevRetrieveMode has the same mode value. If it does not, we need
         * to remove all dfObjects as they have the wrong codeing language for the data flow.
         * 
         * If you just switch between 'retrieveRuby' and 'retrieveJS' diectly we will also clear
         * out the dfObjects that are present since they have the wrong language code. This makes
         * sense to do this as a particular document will have ONE language being used.
        */
        public enum retrieverMode { retrieveRuby, retrieveJS, convertRubyToJS };
       

        public retrieverMode retMode;
        public retrieverMode prevRetrieveMode;

        // This is set to our work directory, the same path on all computers. 
        string curDir = "C:\\temp\\work";

        // ******************************************************************
        // Some Ruby conversion to JS items:

        ConvertRuby convertRuby = null;

        // Debug use only:
        //List<string> exprList;
        //List<string> filterList;
        //List<string> hashList;
        //bool exprHash = true;
        public static bool debug = false;

        // ******************************************************************

        // This is the file that contains the actual dataflow code.
        // This is the real/released path:
        string xmlFile = @"\\stfprdw-apps404\FormsProgramming\Shared Code\Calc Tool Test\dataflow_code.xml";

        // Stores the contents of the snippet xml file. Initially null.
        XDocument xmlDocument;

        // ******************************************************************

        // This is the currently selected tag from the tagList (e.g. "Spouse" for the individual profile).
        string profileTag = null;

        // Indicates which profile is currently selected.
        string profile = null;

        // This is the mnemonic variable we use when retrieving dataflow code from our code snippet file.
        // The value is built as the user selects from the various lists for a given profile.
        // Example of a mnemonic string: IND|INDIVIDUAL:NAME::FIRST
        string mnemonic = null;

        // This holds all info pertaining to the dataflow snippet we are retrieving. This object holds the retrieved data flow,
        // box information, the mnemonic which describes the dataflow snippet we are retrieving, and other pertinent information.
        DataFlow dfObject = null;

        // We use this to lock the tagList when performing common block processing [dependent processing is an example].
        bool tagListLocked = false;

        /*
        # This list holds FormBox objects initialized with box data extracted from the Ruby Source Code file,
        # or the XML file if APF box width info is desired [we also grab the box numbers from the XML].
        */
        List<FormBox> formBoxList = new List<FormBox>();

        /*
        #
        # This list holds the dfObject(s) associated with the 'Selection List'. We renumber the data flow
        # code [during retrieval] using the objects in this list - see class definition for DataFlow.
        #
        */
        List<DataFlow> dfObjectList = new List<DataFlow>();

        /*
        *
        * #####################################################
        *
        * Common Block Processing:
        *
        * Currently handles Individual Profile Dependents and Equifax Officers.
        *
        * This is a twist on the dataflow retrieval logic: The number of dependents/officers must 
        * be specified, and each dependent/officer will have the same dataflow elements - name style [last first, 
        * first last, etc.], dob format, etc.. So there will be a block of dataflow for dependent/officer number 1,
        * 2, etc., up to the appropriate maximum.
        *
        * We will use the appropriate List to hold the dependent/officer dataflow elements [dfObject(s)
        * of the class DataFlow - we grab the contents of dfObjectList] which pertain to all
        * dependents/officers, and display the contents [in the 'Selection List' after converting to the 'box prefix +
        * mnemonic' format] of this list when switching to and from <Dependents>/<Officers> in the appropriate Profile
        * Tag List.
        *
        */
        int numDependents = 1;
        int numEquSuccOfficers = 1;
        int numEquPredOfficers = 1;

        // These hold the box offsets for common block processing. So if a document has dependent 1 name box as 'box1_10'
        // and dependent 2 name box as 'box1_25', the offset would be 15. The retriever user enters these offsets.
        string indivDepOffset;
        string equSuccOfficOffset;
        string equPredOfficOffset;

        List<DataFlow> indivDepMnemonicList = new List<DataFlow>();
        List<DataFlow> equSuccOffMnemonicList = new List<DataFlow>();
        List<DataFlow> equPredOffMnemonicList = new List<DataFlow>();

        /*
        *
        * #####################################################
        *
        */

        // These lists function like the common block lists above in that they store the 'Selection List' mnemonics for
        // a particular profile. This way the user does not lose this info when switching between profiles.
        List<DataFlow> indivMnemonicList = new List<DataFlow>();
        List<DataFlow> equSuccMnemonicList = new List<DataFlow>();
        List<DataFlow> equPredMnemonicList = new List<DataFlow>();
        List<DataFlow> equGenMnemonicList = new List<DataFlow>();
        List<DataFlow> equStateMnemonicList = new List<DataFlow>();
        List<DataFlow> entityMnemonicList = new List<DataFlow>();
        List<DataFlow> prepMnemonicList = new List<DataFlow>();
        List<DataFlow> fidMnemonicList = new List<DataFlow>();

        /*
        ####################################################################################
        # 
        # Arrays Used By More Than One Profile:
        # 
        ####################################################################################
        */
        /*
        * IND|BUSINESS:FEIN
        * FID|ESTATE/TRUST:FEIN
        * EQU SUCCESSOR|OFFICERS:FEIN
        * EQU SUCCESSOR|FEIN
        * EQU SUCCESSOR|PARENT:FEIN
        * EQU PREDECESSOR|FEIN
        */
        string[] feinArray = { "Single Box", "FEIN in 2 boxes" };

        /*
        * IND|BUSINESS:FEIN::SINGLE BOX
        * FID|ESTATE/TRUST:FEIN::SINGLE BOX
        * EQU SUCCESSOR|OFFICERS:FEIN
        * EQU SUCCESSOR|FEIN::SINGLE BOX
        * EQU SUCCESSOR|PARENT:FEIN::SINGLE BOX
        * EQU SUCCESSOR|ENTITY TYPE:DISREGARDED ENTITY::SINGLE MEMBER FEIN
        * EQU SUCCESSOR|ENTITY TYPE:DISREGARDED ENTITY::TWO BOXES: CHECK ['X'] A BOX; SINGLE MEMBER FEIN
        * EQU SUCCESSOR|ENTITY TYPE:DISREGARDED ENTITY::TWO BOXES: SINGLE MEMBER NAME; SINGLE MEMBER FEIN
        * EQU SUCCESSOR|ENTITY TYPE:DISREGARDED ENTITY::THREE BOXES: CHECK ['X'] A BOX; SINGLE MEMBER NAME; SINGLE MEMBER FEIN
        */
        string[] feinManipArray = { "<None>", "999999999", "99 9999999", "Last 4 digits" };

        /*
        * IND|INDIVIDUAL:ADDRESS::ZIP
        * IND|SPOUSE:ADDRESS::ZIP
        * IND|BUSINESS:ADDRESS::ZIP
        * ENT|GENERAL:ADDRESS::ZIP
        * ENT|AGENT:ADDRESS::ZIP
        * FID|FIDUCIARY:ZIP
        * FID|DECEDENT/DONOR:ZIP
        * PRE|ADDRESS::ZIP
        * EQU SUCCESSOR|ADDRESS:CORPORATE::ZIP
        * EQU SUCCESSOR|ADDRESS:MAILING::ZIP
        * EQU PREDECESSOR|CONTACT:ADDRESS::ZIP
        * EQU PREDECESSOR|CORPORATE ADDRESS::ZIP
        */
        string[] zipManipArray = { "<None>", "5 digits: 99999", "9 digits: 999999999", "9 digits: 99999 9999", "9 digits: 99999-9999", "5 digits: 99999 or 9 digits: 999999999", "5 digits: 99999 or 9 digits: 99999-9999", "5 digits: 99999 or 9 digits: 999999999 or 99999 9999", "5 digits: 99999 or 9 digits: 99999-9999 or 99999 - 9999", "2 boxes - 5 digits in first: 99999 4 digits in second: 9999 " };

        /*
        * IND|INDIVIDUAL:PHONE
        * ENT|GENERAL:PHONE
        * ENT|CONTACT:PHONE
        * ENT|GENERAL+CONTACT:PHONE
        * FID|FIDUCIARY:PHONE
        * FID|DECEDENT/DONOR:PHONE
        * PRE|PHONE
        * EQU SUCCESSOR|PAYROLL/TAX CONTACT:PHONE
        * EQU SUCCESSOR|PAYROLL SERVICE PROVIDER:PHONE
        * EQU SUCCESSOR|BANK ACCOUNT INFORMATION
        * EQU PREDECESSOR|CONTACT:PHONE
        */
        string[] phoneManipArray = { "<None>", "10 digits: 9999999999", "10 digits: 999 999 9999", "10 digits: 999-999-9999", "10 digits: 999 999-9999", "10 digits: (999) 999 9999", "10 digits: (999) 999-9999", "10 digits: (999)999-9999", "7 digits: 999-9999 or 10 digits: 999-999-9999", "Two boxes - area code in first: 999 second box: 999 9999", "Two boxes - area code in first: 999 second box: 999-9999", "Two boxes - area code in first: (999) second box: 999 9999 ", "Two boxes - area code in first: (999) second box: 999-9999 ", "Three boxes - area code in first: 999 second box: 999 third box: 9999", "Three boxes - area code in first: (999) second box: 999 third box: 9999" };

        /*
        *
        * IND|SPOUSE:SSN
        * IND|DEPENDENTS:SSN
        * FID|FIDUCIARY:SSN
        * FID|DECEDENT/DONOR:SSN
        * EQU SUCCESSOR|OFFICERS:SSN
        */
        string[] ssnArray = { "Single Box", "SSN in 3 boxes" };
        string[] ssnManipArray = { "<None>", "999999999", "999 99 9999", "Last 4 digits" };

        /*
        *
        * IND|SOLE PROPRIETOR:FEIN / SSN
        * FID|ESTATE/TRUST:FEIN / SSN
        * EQU SUCCESSOR|OFFICERS:SSN/FEIN
        */
        string[] feinSsnArray = { "Single Box", "Single Box with Separate Checkboxes", "Separate Boxes: FEIN in 1 box else SSN in 1 box", "Separate Boxes: FEIN in 2 boxes else SSN in 3 boxes" };
        string[] feinSsnManipArray = { "<None>", "999999999", "99 9999999 or 999 99 9999", "Last 4 digits" };

        /*
        * IND|DEPENDENTS:DOB
        * ENT|GENERAL:DATE STARTED
        * FID|ESTATE/TRUST:CREATION DATE
        * FID|DECEDENT/DONOR:DOB
        * FID|DECEDENT/DONOR:DOD
        * EQU SUCCESSOR|DATE STARTED
        * EQU SUCCESSOR|FISCAL YEAR END DATE
        * EQU SUCCESSOR|FIRST DAY OF EMPLOYMENT
        * EQU SUCCESSOR|FIRST PAYROLL DATE
        * EQU SUCCESSOR|20TH WEEK EMPLOYMENT DATE OF 1 OR MORE
        */
        string[] dateManipArray = { "<None>", "MM/DD/YY", "MM-DD-YY", "MM DD YY", "MMDDYY", "MM/DD/YYYY", "MM-DD-YYYY", "MM DD YYYY", "MMDDYYYY", "Three boxes - MM, DD, YY", "Three boxes - MM, DD, YYYY", "Three boxes - '9th', 'February', 'YYYY' (Ex.)", "Three boxes - '09', 'February', 'YYYY' (Ex.)", "Two boxes - 'February 9th', 'YYYY' (Ex.)" };

        /*
        * EQU SUCCESSOR|SUBJECT TO FUTA:FIRST LIABILITY YEAR
        * EQU SUCCESSOR|SUBJECT TO FUTA:TWO BOXES: CHECK ['X'] YES BOX; FIRST LIABILITY YEAR
        * EQU SUCCESSOR|TOTAL PAYROLL DATE OF $ OR MORE:YEAR
        */
        string[] yearManipArray = { "<None>", "YY" };

        /*
        * ENT|GENERAL:DATE STARTED+STATE OF INCORPORATION
        * EQU SUCCESSOR|DATE STARTED+STATE OF INCORPORATION
        */
        string[] dateStartedStateOfIncArray = { "Date Started + State of Incorporation", "State of Incorporation + Date Started" };

        /*
        * IND|BUSINESS:ADDRESS
        * ENT|GENERAL:ADDRESS
        * EQU SUCCESSOR|ADDRESS:CORPORATE
        * EQU SUCCESSOR|ADDRESS:MAILING
        * EQU PREDECESSOR|CORPORATE ADDRESS
        * EQU PREDECESSOR|CONTACT:ADDRESS
        */
        string[] addressArray = { "Addr1", "Addr2", "Addr1+Addr2", "Addr1+Addr2+City", "Addr1+Addr2+City+State", "Addr1+Addr2+City+State+Zip", "City+State", "City+State+Zip", "State+Zip", "City", "State", "Zip" };

        // ENT|AGENT:ADDRESS
        // FID|FIDUCIARY:ADDRESS
        // FID|DECEDENT/DONOR:ADDRESS
        // PRE|ADDRESS
        // EQU SUCCESSOR|PAYROLL SERVICE PROVIDER:ADDRESS
        // EQU SUCCESSOR|BANK ACCOUNT INFORMATION:ADDRESS
        // EQU SUCCESSOR|OFFICERS:ADDRESS
        // EQU PREDECESSOR|OFFICERS:ADDRESS
        string[] addrArray = { "Address", "Address+City+State", "Address+City+State+Zip", "City", "City+State", "City+State+Zip", "State", "State+Zip", "Zip" };

        /*
        ####################################################################################
        # 
        # Tag Arrays: These are all of the arrays used in populating the various lists.
        # 
        ####################################################################################
        */

        /* 
        ##############################################################
        # 
        # Profile Tag Arrays:
        #
        ##############################################################
        */ 

        // Individual profile Tag array:
        string[] indivProfileArray = { "Individual", "Spouse", "Individual+Spouse", "Business", "Business/Individual", "Business/Individual+Spouse", "Sole Proprietor", "Dependents", "Filing Status", "Direct Deposit" };

        // Entity profile Tag array:
        string[] entProfileArray = { "General", "Contact", "General+Contact", "Agent" };

        // Fiduciary profile Tag array:
        string[] fidProfileArray = { "Fiduciary", "Estate/Trust", "Decedent/Donor", "Estate/Trust+Decedent/Donor" };

        // Preparer profile Tag array:
        string[] preProfileArray = { "Name", "Address", "Name+Address", "Self Employed", "Self Employed+Name", "Self Employed+Name+Address", "ID Number", "Name+ID Number", "Phone", "Name+Phone", "Fax", "Phone+Fax", "Email" };

        // Equifax profile Tag array - This list is displayed when we select the Equifax profile:
        // The 'Profile Tags' list is redrawn and the menmonic is adjusted per the Tag List selection.
        string[] equProfileArray = { "General", "Successor", "Predecessor", "State List" };

        // These Equifax profile Tag arrays are displayed when we select from the equProfileArray:
        // EQU GENERAL
        string[] equGenArray = { "<Return>", "Account Name", "Tax Accounts Required", "Reason For Application" };

        // EQU SUCCESSOR
        string[] equSuccArray = { "<Return>", "Name", "Address", "Name+Address", "FEIN", "Phone", "Payroll/Tax Contact", "Entity Type", "Date Started", "State of Incorporation", "Date Started+State of Incorporation", "Fiscal Year End Date", "Accounting Method", "First Day of Employment", "First Payroll Date", "Nature of Business Activity", "Business Activity Description", "NAICS/SIC Code", "501 (c)(3) Exempt", "Subject to FUTA", "Payroll Service Provider", "Bank Account Information", "Employment in Other States", "Break in Employment", "Total Payroll Date of $ or More", "20th Week Employment Date of 1 or More", "Parent", "Officers", "<Return>" };

        // EQU PREDECESSOR
        string[] equPredArray = { "<Return>", "Name", "FEIN", "Phone", "Corporate Address", "Contact", "Date of Transaction", "Operating at Time of Purchase", "Transaction Type", "All/Part Employment in Each State", "Employment Agency", "Business Activity Description", "Commonly Owned", "Assets Transferred/Acquired", "Continue to Report Wages", "Parent", "Officers", "<Return>" };

        // EQU STATE
        string[] equStateArray = { "<Return>", "AK", "AL", "AR", "AZ", "CA", "CO", "CT", "DC", "DE", "FL", "GA", "HI", "IA", "ID", "IL", "IN", "KS", "KY", "LA", "MA", "MD", "ME", "MI", "MN", "MO", "MS", "MT", "NC", "ND", "NE", "NH", "NJ", "NM", "NV", "NY", "OH", "OK", "OR", "PA", "PR", "RI", "SC", "SD", "TN", "TX", "UT", "VA", "VI", "VT", "WA", "WI", "WV", "WY", "<Return>" };

        // EQU SUCCESSOR|OFFICERS
        // EQU PREDECESSOR|OFFICERS
        string[] equOfficerSubTagArray = { "Name", "Title", "Name+Title", "SSN", "FEIN", "SSN / FEIN", "Address", "Name+Address", "Name+Title+Address", "DOB", "Date of Appointment" };

        string[] equOfficerTagArray = { "1 Officer", "2 Officers", "3 Officers", "<Return>" };

        /*
        # 
        # NOTE: Individual and Spouse tags have the exact same set of arrays: sub-tag, sub-tag combinations, and
        # manipulations, so using just one set of each rather than having many sets of the same values. Decided to keep
        # the 'indivInd' prefix in the various array names even though spouse is using them too.
        # 
        ##############################################################
        # 
        # Sub-Tag Arrays:
        #
        ##############################################################
        # 
        # Individual profile Sub-Tag arrays:
        # 
        # IND|INDIVIDUAL
        */
        string[] indivIndArray = { "Name", "Address", "Name+Address", "Phone", "Name+Phone", "Fax", "Phone+Fax", "SSN", "Occupation", "School Code", "Email", "Blind", "65+" };

        // IND|INDIVIDUAL+SPOUSE
        string[] indivIndSpArray = { "Name", "SSN" };

        // IND|BUSINESS/INDIVIDUAL+SPOUSE
        string[] indivBusIndSpArray = { "Name" };

        // IND|BUSINESS
        string[] indivBusArray = { "Name", "Address", "Name+Address", "Phone", "Name+Phone", "Fax", "Phone+Fax", "FEIN", "Email" };

        // IND|BUSINESS/INDIVIDUAL
        string[] indivBusIndArray = { "Phone" };

        // IND|SOLE PROPRIETOR
        string[] indivSoleArray = { "Name", "Address", "Phone", "Name+Phone", "Fax", "Phone+Fax", "FEIN / SSN", "Email" };

        // IND|FILING STATUS
        string[] indivFileStatusArray = { "Single", "Married Filing Joint", "Married Filing Separate", "Married Filing Joint or Separate", "Head of Household", "Widow(er)" };

        // IND|DIRECT DEPOSIT
        string[] indivDirectDepositArray = { "Routing Number", "Account Number", "Checking Account", "Savings Account" };

        /*
        # 
        ##########################################
        # 
        # Entity profile Sub-Tag arrays:
        # 
        # ENT|GENERAL 
        */
        string[] entGenArray = { "Name", "Address", "Name+Address", "Phone", "Name+Phone", "Fax", "Phone+Fax", "FEIN", "Email", "State ID", "Date Started", "State of Incorporation", "Date Started+State of Incorporation ", "NAICS Code", "Principal Activity" };

        string[] entConArray = { "Name", "Title", "Name+Title", "Phone", "Name+Phone" };

        string[] entGenConArray = { "Name", "Phone", "Name+Phone" };

        string[] entAgentArray = { "Name", "Address", "Name+Address", "Phone", "Name+Phone" };

        /*
        # 
        ##########################################
        # 
        # Equifax profile Sub-Tag arrays:
        */
        // EQU GENERAL|TAX ACCOUNTS REQUIRED
        string[] equGenTaxAcctsReqArray = { "Unemployment Tax", "Local Withholding Tax", "Withholding Tax", "Employment Tax Consulting" };

        // EQU GENERAL|REASON FOR APPLICATION
        string[] equReasonAppArray = { "Opening a Newly Formed Business", "Existing Entity - New in State(s)", "Did You Acquire All/Part of an Existing Business", "Reopening a Closed/Inactive Business", "Changing Ownership Type/Entity Change", "Name Change", "Account Closures", "Other" };

        // EQU SUCCESSOR|NAME
        // EQU PREDECESSOR|NAME
        string[] equSuccPredNameArray = { "Legal Name", "Trade/DBA Name" };

        // EQU SUCCESSOR|ADDRESS
        string[] equSuccAddrNameArray = { "Corporate", "Mailing" };

        // EQU SUCCESSOR|NAME+ADDRESS
        string[] equSuccNameAddressArray = { "Legal Name+Corporate Address", "Legal Name+Mailing Address" };

        // EQU SUCCESSOR|PAYROLL/TAX CONTACT
        string[] equSuccConArray = { "Name", "Title", "Name+Title", "Phone", "Name+Phone", "Fax", "Email" };

        // EQU PREDECESSOR|CONTACT
        string[] equPredConArray = { "Name", "Phone", "Address", "Name+Phone", "Name+Address" };

        // EQU SUCCESSOR|ENTITY TYPE
        string[] equSuccEntTypeArray = { "Corporation (Publicly Traded)", "Corporation (Privately Held)", "Partnership", "Limited Partnership", "S Corporation", "Other", "Limited Liability Company", "Taxed as Corporation", "Taxed as Partnership", "Disregarded Entity", "Sole Proprietorship" };

        // EQU SUCCESSOR|ACCOUNTING METHOD
        string[] equSuccAccMethodArray = { "Cash", "Accrual" };

        // EQU SUCCESSOR|NATURE OF BUSINESS ACTIVITY
        string[] equSuccNatBusActArray = { "Retail", "Wholesale", "Manufacturing", "Other", "Service", "Staffing/Temp/Emply Agt Services", "Professional Employer Organization" };

        // EQU SUCCESSOR|SUBJECT TO FUTA
        string[] equSuccFutaArray = { "Check ['X'] YES box", "Check ['X'] NO box", "First Liability Year", "Two boxes: Check ['X'] YES box; First Liability Year" };

        // EQU SUCCESSOR|PAYROLL SERVICE PROVIDER
        string[] equSuccPaySvcProvArray = { "Check ['X'] N/A box", "Provider Name", "Contact Name", "Contact Name/Provider Name", "Phone", "Address", "Provider Name+Phone", "Provider Name+Address", "Provider Name+Address+Phone" };

        // EQU SUCCESSOR|BANK ACCOUNT INFORMATION
        string[] equSuccBankAccInfoArray = { "Bank Name", "Address", "Phone", "Bank Name+Address", "Bank Name+Phone", "Bank Name+Address+Phone", "Account Number" };

        // EQU SUCCESSOR|EMPLOYMENT IN OTHER STATES
        string[] equSuccEmpOthStArray = { "YES", "Check ['X'] NO box", "Multiple States" };

        // EQU SUCCESSOR|TOTAL PAYROLL DATE OF $ OR MORE
        string[] equSuccTotPayArray = { "Quarter", "Year", "Quarter+Year" };

        // EQU SUCCESSOR|PARENT
        // EQU PREDECESSOR|PARENT
        string[] equSuccPredParentArray = { "Parent Name", "FEIN" };

        // EQU PREDECESSOR|TRANSACTION TYPE
        string[] equPredTranTypeArray = { "Asset Purchase", "Stock Purchase", "Company Reorganization", "Salutory Merger", "Other", "Employee Reassignment Only", "Bankruptcy Acquisition", "Entity Conversion" };

        // EQU PREDECESSOR|ALL/PART EMPLOYMENT IN EACH STATE
        string[] equPredAllPartArray = { "All", "Partial" };

        // EQU PREDECESSOR|COMMONLY OWNED
        string[] equPredCommOwnedArray = { "Check ['X'] YES box", "Check ['X'] NO box", "Percentage", "Relationship", "Two boxes: Check ['X'] YES box; Percentage", "Two boxes: Check ['X'] YES box; Relationship", "Three boxes: Check ['X'] YES box; Percentage; Relationship" };

        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED
        string[] equPredAssTranAcqArray = { "Accounts Receivable", "Machinery or Equipment", "Furniture or Fixtures", "Goodwill", "Buildings", "Stock", "Other Real Property or Leases", "Customer Lists", "Employees", "Inventories" };

        // EQU PREDECESSOR|CONTINUE TO REPORT WAGES
        string[] equPredContRptWagesArray = { "Check ['X'] YES box", "Check ['X'] NO box", "Last Date of Employment", "Date of Last Payroll", "Check ['X'] NO box; Last Date of Employment", "Check ['X'] NO box; Date of Last Payroll", "Check ['X'] NO box; Last Date of Employment; Date of Last Payroll" };

        // Note: The ## is a placeholder for the actual state abbreviations. These are used in the mnemonics
        // and in the actual DF code.
        //
        // EQU STATE LIST|##
        string[] equStateSubTagArray = { "Account Numbers", "Location Address", "Number Of Employees", "Withholding Tax" };

        /*
        ##########################################
        # 
        # Fiduciary profile Sub-Tag arrays:
        # 
        # FID|FIDUCIARY
        */
        string[] fidFidArray = { "Name","Address","Name+Address","Title","Name+Title","Phone","Name+Phone","SSN" };

        // FID|ESTATE/TRUST
        string[] fidEsTrArray = { "Name","FEIN","SSN","FEIN / SSN","Creation Date","Type" };

        // FID|DECEDENT/DONOR
        string[] fidDecDnArray = { "Name","Address","Name+Address","Phone","Name+Phone","SSN","DOB","DOD" };

        // FID|ESTATE/TRUST+DECEDENT/DONOR
        string[] fidEsTrDecDnArray = { "DOD / Creation Date" };

        /*
        ##########################################
        #
        # Preparer profile Sub-Tag arrays:
        #
        # PRE|NAME
        */
        string[] preNameArray = { "Individual","Firm","Firm / Individual" };

        // PRE|NAME+ADDRESS
        string[] preNameAddressArray = { "[Firm / Individual]+Address" };

        // PRE|SELF EMPLOYED+NAME
        string[] preSempNameArray = { "Name Box","Name Box; Check Box: If Self Employed","Name Box[Prep.]; Name Box[Firm / Prep. If Self Emp.]; Check Box: If Self Emp." };

        // PRE|SELF EMPLOYED+NAME+ADDRESS
        string[] preSempNameAddrArray = { "Name+Address Box","Name+Address Box; Check Box: If Self Employed" };

        // PRE|ID NUMBER
        string[] preIdArray = { "PTIN","FEIN","SSN","PTIN / FEIN / SSN","PTIN / FEIN","PTIN / SSN","FEIN / PTIN","FEIN / SSN" };

        // PRE|NAME+ID NUMBER
        string[] preNameIdArray = { "[Firm / Individual]+ID Number" };

        // PRE|NAME+PHONE
        string[] preNamePhoneArray = { "Individual+Phone", "Firm Name+Phone", "[Firm / Individual]+Phone" };

        /*
        ##############################################################
        #
        # Combination/Manipulation Arrays - Individual Profile:
        #
        # NOTE: Some other profiles may use these arrays and will be duly noted.
        #
        ##############################################################
        */
        // IND|INDIVIDUAL:NAME
        string[] indivIndNameArray = { "First","MI","Last","Suffix","First+MI+Last+Suffix","First+MI+Suffix+Last","First+Last+MI", "Last+First+MI","First+MI+Last", "Last+Suffix+First","First+MI", "First+Last", "Last+First" };

        // IND|INDIVIDUAL:ADDRESS
        string[] indivIndAddressArray = { "Addr","Addr+Apt","Addr+Apt+City","Addr+Apt+City+County","Addr+Apt+City+State","Addr+Apt+City+State+County","Addr+Apt+City+State+Zip","Addr+Apt+City+Zip","Addr+Apt+State+Zip","Addr+Apt+Zip","Apt","Apt Num","City","City+County+State","City+State","City+State+Zip","City+State+Zip+County","City+Zip","County","County+State","State","State+Zip","Zip" };

        // IND|INDIVIDUAL:NAME+ADDRESS
        string[] indivIndNameAddressArray = { "First+MI+Last+Addr+Apt+City+State+Zip","First+MI+Last+Suffix+Addr+Apt+City+State+Zip" };

        // IND|BUSINESS:NAME+ADDRESS
        string[] indivBusNameAddressArray = { "Name+Addr1+Addr2+City+State+Zip" };

        /*
        # Made a separate array for ENT|GENERAL because there are forms that want a SSN if an Individual,
        # or, if an Entity like a corporation is the tax payer, they want an FEIN, so one box would hold either a SSN
        # or an FEIN, but they have 2 checkboxes on the form to indicate if the ID Number is a SSN or an FEIN.
        # May possibly just do this for all SSN or FEIN situations for all profiles, in which case we would just
        # have 1 feinArray and that would cover it all, but for now I cannot see the sense in doing it.
        #
        # ENT|GENERAL:FEIN
        */
        string[] entFeinArray = { "Single Box","Single Box with Separate Checkbox","FEIN in 2 boxes" };

        // IND|INDIVIDUAL:PHONE
        string[] indivIndPhoneArray = { "Home","Work","Mobile","Home/Work","Home/Mobile","Work/Home/Mobile" };

        // IND|INDIVIDUAL:NAME+PHONE
        string[] indivIndNamePhoneArray = { "[First+MI+Last+Suffix]+[Work/Home/Mobile]" };

        // IND|INDIVIDUAL:PHONE+FAX
        string[] indivIndPhoneFaxArray = { "Home+Fax","Work+Fax","Mobile+Fax","[Home/Work]+Fax","[Home/Mobile]+Fax","[Work/Home/Mobile]+Fax" };

        /*
        # Made a separate array for INDIVIDUAL because there are forms that want a SSN if an Individual,
        # or, if an Entity like a corporation is the tax payer, they want an FEIN, so one box would hold either a SSN
        # or an FEIN, but they have 2 checkboxes on the form to indicate if the ID Number is a SSN or an FEIN.
        # May possibly just do this for all SSN or FEIN situations for all profiles, in which case we would just
        # have 1 ssnArray and that would cover it all, but for now I cannot see the sense in doing it.
        #
        # IND|INDIVIDUAL:SSN
        */
        string[] indSsnArray = { "Single Box","Single Box with Separate Checkbox","SSN in 3 boxes" };

        //# ################################################

        // IND|INDIVIDUAL:BLIND
        string[] blindManipArray = { "Check ['X'] a box if blind" };

        // IND|INDIVIDUAL:65+
        string[] sixtyFivePlusManipArray = { "Check ['X'] a box if age 65 or over" };

        // IND|INDIVIDUAL+SPOUSE:NAME
        string[] indivIndSpNameArray = { "First+MI+Last [ & if joint: SPFirst+SPMI+SPLast]", "First+MI+Last [ & if SPFirst: SPFirst+SPMI+SPLast]", "First+MI [ & if SPFirst: SPFirst+SPMI+SPLast(if different)]+Last", "First+MI+Last+Suffix [ & if joint: SPFirst+SPMI+SPLast+SPSuffix]", "First+MI+Last+Suffix [ & if SPFirst: SPFirst+SPMI+SPLast+SPSuffix]", "First+MI [ & if joint: SPFirst+SPMI]", "First+MI [ & if joint: SPFirst+SPMI]+Last", "First+MI [ & if joint: SPFirst+SPLast(if different)]+Last", "First+Last [ & if SPFirst: SPFirst+SPLast]", "Last+First+MI [ & if joint: SPLast+SPFirst+SPMI]", "Last+First+MI [ & if joint: SPLast(if different)+SPFirst+SPMI]", "Last+First+MI [ & if joint: SPFirst+SPMI]", "Last+First+MI [ & if SPFirst: SPLast+SPFirst+SPMI]", "Last+Suffix+First+MI [ & if joint: SPLast+SPSuffix+SPFirst+SPMI]" };

        // IND|INDIVIDUAL+SPOUSE:SSN
        string[] indivIndSpSsnArray = { "SSN + SPSSN", "SSN [ & if joint: SPSSN]" };

        // IND|BUSINESS/INDIVIDUAL:PHONE
        string[] indivBusIndPhoneArray = { "Business/Work" };

        // IND|BUSINESS/INDIVIDUAL+SPOUSE:NAME
        string[] indivBusIndSpNameArray = { "Business / First+MI [ & if joint: SPFirst+SPMI]+Last " };

        // IND|SOLE PROPRIETOR:NAME
        string[] indivSoleNameArray = { "Business / First+MI+Last","Business / First+MI+Last+Suffix","Business / Last+First+MI" };

        // IND|SOLE PROPRIETOR:ADDRESS
        string[] indivSoleAddressArray = { "Addr1; Addr2; City; State; Zip","Addr1; Addr2; City; State+Zip","Addr1; Addr2; City+State; Zip","Addr1; Addr2; City+State+Zip","Addr; Apt; City+State+Zip","Addr; City; County; State; Zip","Addr; City; County; State+Zip","Addr; City; State; Zip","Addr; City; State+Zip","Addr; City; Zip","Addr; City+State; Zip","Addr; City+State+Zip","Addr; City+State+Zip; County","Addr+City; County; State","Addr+City; State; Zip","Addr+City+State; Zip","Addr+City+State+Zip","Addr+City+State+Zip+County" };

        // IND|SOLE PROPRIETOR:PHONE
        string[] indivSolePhoneArray = { "Business/Work/Home/Mobile" };

        // IND|SOLE PROPRIETOR:NAME+PHONE
        string[] indivSoleNamePhoneArray = { "[First+MI+Last+Suffix]+[Business/Work/Home/Mobile]","[Business Name]+[Business/Work/Home/Mobile]","[Business Name / First+MI+Last+Suffix]+[Business/Work/Home/Mobile]" };

        // IND|SOLE PROPRIETOR:PHONE+FAX
        string[] indivSolePhoneFaxArray = { "[Business/Work/Home/Mobile]+Fax" };

        /*
        # IND|DEPENDENTS
        #
        # The Dependents data flow code retrieval process has a twist in that there are a total of 6 dependents in the 
        # Individual profile. The data flow coding is always identical on a form for a multiple  number of dependents. So
        # a form would want say, a name [expressed as say, 'First Last'] and perhaps a SSN expressed as '999 99 9999'
        # and a relationship - this would be the same info wanted for dependent #1 up to how many dependents are listed
        # on the form. This is what is referred to as Common Block processing.
        #
        # Given that, and the way this retrieval tool is designed, we will re-populate the Profile Tag list with a 
        # dependent processing array where the user can mouse click on the number of dependents to process or <Return>
        # to the deault Individual Profile Tag list contents.
        */
        string[] indivDepArray = { "Name","SSN","Relationship","DOB" };

        string[] indivDepProcArray = { "1 Dependent","2 Dependents","3 Dependents","4 Dependents","5 Dependents","6 Dependents","<Return>" };

        // IND|DEPENDENTS:NAME
        string[] indivDepNameArray = { "First", "Last", "First+Last", "Last+First" };

        // IND|FILING STATUS:SINGLE
        string[] singleManipArray = { "Check ['X'] a box if single" };

        // IND|FILING STATUS:MARRIED FILING JOINT
        string[] jointManipArray = { "Check ['X'] a box", "Check ['X'] a box if married filing joint" };

        // IND|FILING STATUS:MARRIED FILING SEPARATE
        string[] separateManipArray = { "Check ['X'] a box", "Check ['X'] a box if married filing separate" };

        // IND|FILING STATUS:MARRIED FILING JOINT OR SEPARATE
        string[] jointSeparateManipArray = { "Check ['X'] a box", "Check ['X'] a box if married filing joint or separate" };

        // IND|FILING STATUS:HEAD OF HOUSEHOLD
        string[] hohManipArray = { "Check ['X'] a box if head of household" };

        // IND|FILING STATUS:WIDOW(ER)
        string[] widowManipArray = { "Check ['X'] a box if widow(er)" };

        // IND|DIRECT DEPOSIT:CHECKING
        string[] ddCkManipArray = { "Check ['X'] a box if checking account" };

        // IND|DIRECT DEPOSIT:SAVING
        string[] ddSvManipArray = { "Check ['X'] a box if savings account" };

        /*
        ##############################################################
        #
        # Combination/Manipulation Arrays - Entity Profile:
        #
        ##############################################################
        */
        // ENT|GENERAL:NAME+ADDRESS
        string[] entGenNameAddressArray = { "Name+Addr1+Addr2+City+State+Zip" };

        // ENT|GENERAL+CONTACT:NAME
        string[] entGenConNameArray = { "Contact Name / Entity Name" };

        // ENT|GENERAL+CONTACT:PHONE
        string[] entGenConPhoneArray = { "Contact Phone / Entity Phone" };

        // ENT|GENERAL+CONTACT:NAME+PHONE
        string[] entGenConNamePhoneArray = { "Contact Name + [Contact Phone / Entity Phone] ","[Contact Phone / Entity Phone] + Contact Name " };

        // ENT|AGENT:NAME
        string[] entAgentNameArray = { "First","MI","Last","First+MI+Last","Business","Business / First+MI+Last" };

        // ENT|AGENT:NAME+ADDRESS
        string[] entAgentNameAddressArray = { "Business+Address+City+State+Zip","First+MI+Last+Address+City+State+Zip","[Business / First+MI+Last]+Address+City+State+Zip" };

        // ENT|AGENT:NAME+PHONE
        string[] entAgentNamePhoneArray = { "[First+MI+Last] + Phone","Business Name + Phone","[Business Name / First+MI+Last] + Phone" };

        /*
        ##############################################################
        #
        # Combination/Manipulation Arrays - Equifax Profile:
        #
        ##############################################################
        */
        // EQU PREDECESSOR|CONTACT:NAME+ADDRESS
        string[] equPredConNameAddrArray = { "Name+Addr1+Addr2+City+State+Zip" };

        // EQU SUCCESSOR|PAYROLL SERVICE PROVIDER:PROVIDER NAME+ADDRESS
        // EQU SUCCESSOR|BANK ACCOUNT INFORMATION:BANK NAME+ADDRESS
        // EQU SUCCESSOR|OFFICERS:NAME+ADDRESS
        // EQU PREDECESSOR|OFFICERS:NAME+ADDRESS
        string[] equNameAddrArray = { "Name+Address+City+State+Zip" };

        // EQU SUCCESSOR|PAYROLL SERVICE PROVIDER:PROVIDER NAME+ADDRESS+PHONE
        // EQU SUCCESSOR|BANK ACCOUNT INFORMATION:BANK NAME+ADDRESS+PHONE
        string[] equNameAddrPhoneArray = { "Name+Address+City+State+Zip+Phone" };

        // EQU SUCCESSOR|EMPLOYMENT IN OTHER STATES:YES
        string[] equSuccEmpOthStYesArray = { "Check ['X'] YES box", "Multiple States", "Two boxes: Check ['X'] YES box; Multiple States" };

        // EQU SUCCESSOR|OFFICERS:NAME+TITLE+ADDRESS
        // EQU PREDECESSOR|OFFICERS:NAME+TITLE+ADDRESS
        string[] equNameTitleAddrArray = { "Name+Title+Address+City+State+Zip" };

        // EQU GENERAL|TAX ACCOUNTS REQUIRED:UNEMPLOYMENT TAX
        // EQU GENERAL|TAX ACCOUNTS REQUIRED:LOCAL WITHHOLDING TAX
        // EQU GENERAL|TAX ACCOUNTS REQUIRED:WITHHOLDING TAX
        // EQU GENERAL|TAX ACCOUNTS REQUIRED:EMPLOYMENT TAX CONSULTING
        // EQU SUCCESSOR|501 (C)(3) EXEMPT
        // EQU SUCCESSOR|BREAK IN EMPLOYMENT
        string[] yesNoChkBoxArray = { "Check ['X'] YES box", "Check ['X'] NO box" };

        // EQU GENERAL|REASON FOR APPLICATION:OPENING A NEWLY FORMED BUSINESS
        // EQU GENERAL|REASON FOR APPLICATION:EXISTING ENTITY - NEW IN STATE(S)
        // EQU GENERAL|REASON FOR APPLICATION:DID YOU ACQUIRE ALL/PART OF AN EXISTING BUSINESS
        // EQU GENERAL|REASON FOR APPLICATION:REOPENING A CLOSED/INACTIVE BUSINESS
        // EQU GENERAL|REASON FOR APPLICATION:CHANGING OWNERSHIP TYPE/ENTITY CHANGE
        // EQU GENERAL|REASON FOR APPLICATION:NAME CHANGE
        // EQU GENERAL|REASON FOR APPLICATION:ACCOUNT CLOSURES
        // EQU SUCCESSOR|ENTITY TYPE:CORPORATION (PUBLICLY TRADED)
        // EQU SUCCESSOR|ENTITY TYPE:CORPORATION (PRIVATELY HELD)
        // EQU SUCCESSOR|ENTITY TYPE:PARTNERSHIP
        // EQU SUCCESSOR|ENTITY TYPE:LIMITED PARTNERSHIP
        // EQU SUCCESSOR|ENTITY TYPE:S CORPORATION
        // EQU SUCCESSOR|ENTITY TYPE:LIMITED LIABILITY COMPANY
        // EQU SUCCESSOR|ENTITY TYPE:TAXED AS CORPORATION
        // EQU SUCCESSOR|ENTITY TYPE:TAXED AS PARTNERSHIP
        // EQU SUCCESSOR|ENTITY TYPE:SOLE PROPRIETORSHIP
        // EQU PREDECESSOR|TRANSACTION TYPE:ASSET PURCHASE
        // EQU PREDECESSOR|TRANSACTION TYPE:STOCK PURCHASE
        // EQU PREDECESSOR|TRANSACTION TYPE:COMPANY REORGANIZATION
        // EQU PREDECESSOR|TRANSACTION TYPE:SALUTORY MERGER
        // EQU PREDECESSOR|TRANSACTION TYPE:EMPLOYEE REASSIGNMENT ONLY
        // EQU PREDECESSOR|TRANSACTION TYPE:BANKRUPTCY ACQUISITION
        // EQU PREDECESSOR|TRANSACTION TYPE:ENTITY CONVERSION
        // EQU PREDECESSOR|ALL/PART EMPLOYMENT IN EACH STATE:ALL
        // EQU PREDECESSOR|ALL/PART EMPLOYMENT IN EACH STATE:PARTIAL
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:ACCOUNTS RECEIVABLE
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:MACHINERY OR EQUIPMENT
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:FURNITURE OR FIXTURES
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:GOODWILL
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:BUILDINGS
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:STOCK
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:OTHER REAL PROPERTY OR LEASES
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:CUSTOMER LISTS
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:EMPLOYEES
        // EQU PREDECESSOR|ASSETS TRANSFERRED/ACQUIRED:INVENTORIES
        string[] chkBoxArray = { "Check ['X'] a box" };

        // EQU GENERAL|REASON FOR APPLICATION:OTHER
        // EQU SUCCESSOR|ENTITY TYPE:OTHER
        // EQU PREDECESSOR|TRANSACTION TYPE:OTHER
        string[] equOtherArray = { "Check ['X'] a box", "Description", "Two boxes: Check ['X'] a box; Description" };

        // EQU SUCCESSOR|NAME+ADDRESS:LEGAL NAME+CORPORATE ADDRESS
        // EQU SUCCESSOR|NAME+ADDRESS:LEGAL NAME+MAILING ADDRESS
        string[] equSuccLegalNameAddressArray = { "Legal Name+Addr1+Addr2+City+State+Zip" };

        // EQU SUCCESSOR|ENTITY TYPE:DISREGARDED ENTITY
        string[] equDisEntArray = { "Check ['X'] a box", "Single Member Name", "Single Member FEIN", "Two boxes: Single Member FEIN", "Two boxes: Check ['X'] a box; Single Member Name", "Two boxes: Check ['X'] a box; Single Member FEIN", "Three boxes: Check ['X'] a box; Single Member FEIN in 2 boxes", "Two boxes: Single Member Name; Single Member FEIN", "Three boxes: Single Member Name; Single Member FEIN in 2 boxes", "Three boxes: Check ['X'] a box; Single Member Name; Single Member FEIN", "Four boxes: Check ['X'] a box; Single Member Name; Single Member FEIN in 2 boxes" };

        // Note: The ## is a placeholder for the actual state abbreviations.
        //
        // EQU STATE LIST|##:ACCOUNT NUMBERS
        string[] equStateAcctNumArray = { "Predecessor Unemployment", "Successor Unemployment", "Predecessor Withholding" };

        // EQU STATE LIST|##:NUMBER OF EMPLOYEES
        string[] equStateNumEmpArray = { "Transferred To Successor Employer", "Retained By Predecessor Employer" };

        // EQU STATE LIST|##:WITHHOLDING TAX
        string[] equStateWithTaxArray = { "Estimated Monthly Wages For Successor", "Estimated State" };

        /*
        ##############################################################
        #
        # Combination/Manipulation Arrays - Fiduciary Profile:
        #
        ##############################################################
        */
        // FID|FIDUCIARY:NAME
        // FID|DECEDENT/DONOR:NAME
        string[] fidNameArray = { "First_MI","Last","First_MI+Last","Last+First_MI" };

        // FID|FIDUCIARY:NAME+ADDRESS
        // FID|DECEDENT/DONOR:NAME+ADDRESS
        string[] fidNameAddressArray = { "First_MI+Last+Address+City+State+Zip" };

        // FID|FIDUCIARY:NAME+TITLE
        string[] fidFidNameTitleArray = { "First_MI+Last+Title" };

        // FID|FIDUCIARY:NAME+PHONE
        // FID|DECEDENT/DONOR:NAME+PHONE
        string[] fidFidNamePhoneArray = { "[First_MI+Last] + Phone" };

        // FID|ESTATE/TRUST:TYPE
        string[] fidEsTrTypeManipArray = { "Check ['X'] a box if Decedent's Estate","Check ['X'] a box if Simple Trust","Check ['X'] a box if Complex Trust","Check ['X'] a box if Qualified Disability Trust","Check ['X'] a box if ESBT (S portion only)","Check ['X'] a box if Grantor Type Trust","Check ['X'] a box if Bankruptcy Estate - Chapter 7","Check ['X'] a box if Bankruptcy Estate - Chapter 11","Check ['X'] a box if Bankruptcy Estate: Ch. 7 or Ch. 11","Check ['X'] a box if Pooled Income Fund","Check ['X'] a box if: Qualified Disability Trust or Pooled Income Fund or ESBT (S portion only)" };

        /*
        ##############################################################
        #
        # Combination/Manipulation Arrays - Preparer Profile:
        #
        ##############################################################
        */
        // PRE|NAME:INDIVIDUAL
        string[] preIndNameCombArray = { "First","MI","Last","First+MI+Last" };

        // PRE|NAME:FIRM / INDIVIDUAL
        string[] preFirmIndNameCombArray = { "Firm Name / First+MI+Last" };

        // PRE|NAME+ADDRESS:[FIRM / INDIVIDUAL]+ADDRESS
        string[] preFirmIndNameAddrCombArray = { "[Firm Name / First+MI+Last]+Address+City+State+Zip" };

        // PRE|SELF EMPLOYED:::CHECK ['X'] A BOX IF SELF EMPLOYED
        string[] preSempManipArray = { "Check ['X'] a box if self employed" }; 

        // PRE|SELF EMPLOYED+NAME:NAME BOX
        string[] preSempNameBoxArray = { "[If Self Emp.:First+MI+Last] / Firm Name" }; 

        // PRE|SELF EMPLOYED+NAME:NAME BOX; CHECK BOX: IF SELF EMPLOYED
        string[] preSempNameChkBoxArray = { "[If Self Emp.:First+MI+Last; Check a box['X']] / Firm Name " }; 

        // PRE|SELF EMPLOYED+NAME:NAME BOX[PREP.]; NAME BOX[FIRM / PREP. IF SELF EMP.] ; CHECK BOX: IF SELF EMPLOYED
        string[] preSempNamesChkBoxArray = { "First+MI+Last; [If Self Emp.:First+MI+Last; Check a box['X']] / Firm Name " }; 

        // PRE|SELF EMPLOYED+NAME+ADDRESS:NAME+ADDRESS BOX
        string[] preSempNameAddrBoxArray = { "[[If Self Emp.:First+MI+Last] / Firm Name] + Address+City+State+Zip" }; 

        // PRE|SELF EMPLOYED+NAME+ADDRESS:NAME+ADDRESS BOX; CHECK BOX: IF SELF EMPLOYED
        string[] preSempNameAddrChkBoxArray = { "[[If Self Emp.:First+MI+Last; Check a box['X']] / Firm Name] + Address+City+State+Zip" }; 

        // PRE|ID NUMBER:PTIN / FEIN / SSN
        string[] ptinFeinSsnArray= { "Single Box", "Single Box with Separate Checkboxes", "Separate Boxes: PTIN in 1 box else FEIN in 1 box else SSN in 1 box","Separate Boxes: PTIN in 1 box else FEIN in 2 boxes else SSN in 3 boxes" };

        string[] ptinFeinSsnManipArray = { "<None>","P99999999 or 999999999","P99999999 or 99 9999999 or 999 99 9999" };

        // PRE|ID NUMBER:PTIN / FEIN
        string[] ptinFeinArray= { "Single Box", "Single Box with Separate Checkboxes", "Separate Boxes: PTIN in 1 box else FEIN in 1 box","Separate Boxes: PTIN in 1 box else FEIN in 2 boxes" };

        string[] ptinFeinManipArray= { "<None>","P99999999 or 999999999","P99999999 or 99 9999999" };

        // PRE|ID NUMBER:PTIN / SSN
        string[] ptinSsnArray= { "Single Box", "Single Box with Separate Checkboxes", "Separate Boxes: PTIN in 1 box else SSN in 1 box","Separate Boxes: PTIN in 1 box else SSN in 3 boxes" };

        string[] ptinSsnManipArray= { "<None>","P99999999 or 999999999","P99999999 or 999 99 9999" };

        // PRE|ID NUMBER:FEIN / PTIN
        string[] feinPtinArray= { "Single Box", "Single Box with Separate Checkboxes", "Separate Boxes: FEIN in 1 box else PTIN in 1 box","Separate Boxes: FEIN in 2 boxes else PTIN in 1 box" };

        string[] feinPtinManipArray= { "<None>","999999999 or P99999999","99 9999999 or P99999999" };

        // PRE|NAME+ID NUMBER:[FIRM / INDIVIDUAL]+ID NUMBER
        string[] preNameIdCombArray = { "[Firm Name / First+MI+Last]+[PTIN / FEIN / SSN] " };

        // PRE|PHONE
        string[] prePhoneCombArray = { "Home","Work","Mobile","Home/Work","Home/Mobile","Work/Home/Mobile" };

        // PRE|NAME+PHONE:INDIVIDUAL+PHONE
        string[] preNamePhoneIndArray = { "[First+MI+Last] + Phone" };

        // PRE|NAME+PHONE:[FIRM / INDIVIDUAL]+PHONE
        string[] preNamePhoneFirmIndArray = { "[Firm Name / First+MI+Last] + Phone" };

        // PRE|PHONE+FAX
        string[] prePhoneFaxCombArray = { "Home+Fax","Work+Fax","Mobile+Fax","[Home/Work]+Fax","[Home/Mobile]+Fax","[Work/Home/Mobile]+Fax" };

        /*
        # ###############################################################
        #
        # Conditional Arrays:
        #
        # ###############################################################
        #
        # Individual Profile - Spouse tag:
        */
        string[] spouseCondArray = { "if (married_joint)", "if (married_separate)", "if (married_joint OR married_separate)" };

        // Individual Profile - Sole Proprietor tag:
        string[] soleCondArray = { "if (business name)" };

        // Fiduciary Profile - all tags: CURRENTLY NOT BEING USED.
        // fid_cond_array = ["if (a Decedent Estate type) end","if (a Simple Trust type) end","if (a Complex Trust type) end","if (a Qualified Disability Trust type) end","if (an ESBT (S portion only) type) end","if (a Grantor Type Trust type) end","if (a Bankruptcy Estate - Chapter 7 type) end","if Bankruptcy Estate - Chapter 11 type) end","if (a Bankruptcy Estate: Ch. 7 or Ch. 11 type) end","if ( a Pooled Income Fund type) end","if (a Qualified Disability Trust or Pooled Income Fund or ESBT (S portion only) type) end"]

    }
}